//
//  Color+Flat.swift
//  Mechanica
//
//  Copyright © 2016-2017 Tinrobots.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation

extension Color {

  /// **Mechanica**
  ///
  /// Flat colors
  /// Color palette from [flatuicolors.com](https://flatuicolors.com)
  public enum Flat {

    // MARK: -

    /// **Mechanica**
    ///
    /// Turquoise color.
    ///
    /// `#1abc9c`
    ///
    /// `rgba(26, 188, 156, 1.0)`
    public static let turquoise      : Color = #colorLiteral(red: 0.1019607843, green: 0.737254902, blue: 0.6117647059, alpha: 1)

    /// **Mechanica**
    ///
    /// Green Sea color.
    ///
    /// `#16a085`
    ///
    /// `rgba(22, 160, 133, 1.0)`
    public static let greenSea       : Color = #colorLiteral(red: 0.0862745098, green: 0.6274509804, blue: 0.5215686275, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Nephritis color.
    ///
    /// `#27ae60`
    ///
    /// `rgba(39, 174, 96, 1.0)`
    public static let nephritis      : Color = #colorLiteral(red: 0.1529411765, green: 0.6823529412, blue: 0.3764705882, alpha: 1)

    /// **Mechanica**
    ///
    /// Emerald color.
    ///
    /// `#2ecc71`
    ///
    /// `rgba(46, 204, 113, 1.0)`
    public static let emerald        : Color = #colorLiteral(red: 0.1803921569, green: 0.8, blue: 0.4431372549, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Belize Hole color.
    ///
    /// `#2980b9`
    ///
    /// `rgba(41, 128, 185, 1.0)`
    public static let belizeHole     : Color = #colorLiteral(red: 0.1607843137, green: 0.5019607843, blue: 0.7254901961, alpha: 1)

    /// **Mechanica**
    ///
    /// Peter River color.
    ///
    /// `#3498db`
    ///
    /// `rgba(52, 152, 219, 1.0)`
    public static let peterRiver     : Color = #colorLiteral(red: 0.2039215686, green: 0.5960784314, blue: 0.8588235294, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Wisteria color.
    ///
    /// `#8e44ad`
    ///
    /// `rgba(142, 68, 173, 1.0)`
    public static let wisteria       : Color = #colorLiteral(red: 0.5568627451, green: 0.2666666667, blue: 0.6784313725, alpha: 1)

    /// **Mechanica**
    ///
    /// Amethyst color.
    ///
    /// `#9b59b6`
    ///
    /// `rgba(155, 89, 182, 1.0)`
    public static let amethyst       : Color = #colorLiteral(red: 0.6078431373, green: 0.3490196078, blue: 0.7137254902, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Midnight Blue color.
    ///
    /// `#2c3e50`
    ///
    /// `rgba(44, 62, 80, 1.0)`
    public static let  midnightBlue  : Color = #colorLiteral(red: 0.1725490196, green: 0.2431372549, blue: 0.3137254902, alpha: 1)

    /// **Mechanica**
    ///
    /// Wet Asphalt color.
    ///
    /// `#34495e`
    ///
    /// `rgba(52, 73, 94, 1.0)`
    public static let  wetAsphalt    : Color = #colorLiteral(red: 0.2039215686, green: 0.2862745098, blue: 0.368627451, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Sun Flower color.
    ///
    /// `#f1c40f`
    ///
    /// `rgba(241, 196, 15, 1.0)`
    public static let  sunFlower     : Color = #colorLiteral(red: 0.9450980392, green: 0.768627451, blue: 0.05882352941, alpha: 1)

    /// **Mechanica**
    ///
    /// Orange color.
    ///
    /// `#f39c12`
    ///
    /// `rgba(243, 156, 18, 1.0)`
    public static let  orange        : Color = #colorLiteral(red: 0.9529411765, green: 0.6117647059, blue: 0.07058823529, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Carrot color.
    ///
    /// `#e67e22`
    ///
    /// `rgba(230, 126, 34, 1.0)`
    public static let  carrot        : Color = #colorLiteral(red: 0.9019607843, green: 0.4941176471, blue: 0.1333333333, alpha: 1)

    /// **Mechanica**
    ///
    /// Pumpkin color.
    ///
    /// `#d35400`
    ///
    /// `rgba(211, 84, 0, 1.0)`
    public static let  pumpkin       : Color = #colorLiteral(red: 0.8274509804, green: 0.3294117647, blue: 0, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Alizarin color.
    ///
    /// `#e74c3c`
    ///
    /// `rgba(231, 76, 60, 1.0)`
    public static let  alizarin      : Color = #colorLiteral(red: 0.9058823529, green: 0.2980392157, blue: 0.2352941176, alpha: 1)

    /// **Mechanica**
    ///
    /// Pomegranate color.
    ///
    /// `#c0392b`
    ///
    /// `rgba(192, 57, 43, 1.0)`
    public static let  pomegranate   : Color = #colorLiteral(red: 0.7529411765, green: 0.2235294118, blue: 0.168627451, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Clouds color.
    ///
    /// `#ecf0f1`
    ///
    /// `rgba(236, 240, 241, 1.0)`
    public static let  clouds        : Color = #colorLiteral(red: 0.9254901961, green: 0.9411764706, blue: 0.9450980392, alpha: 1)

    /// **Mechanica**
    ///
    /// Silver color.
    ///
    /// `#bdc3c7`
    ///
    /// `rgba(189, 195, 199, 1.0)`
    public static let  silver        : Color = #colorLiteral(red: 0.7411764706, green: 0.7647058824, blue: 0.7803921569, alpha: 1)

    // MARK: -

    /// **Mechanica**
    ///
    /// Concrete color.
    ///
    /// `#95a5a6`
    ///
    /// `rgba(149, 165, 166, 1.0)`
    public static let  concrete      : Color = #colorLiteral(red: 0.5843137255, green: 0.6470588235, blue: 0.6509803922, alpha: 1)

    /// **Mechanica**
    ///
    /// Asbestos color.
    ///
    /// `#7f8c8d`
    ///
    /// `rgba(127, 140, 141, 1.0)`
    public static let  asbestos      : Color = #colorLiteral(red: 0.4980392157, green: 0.5490196078, blue: 0.5529411765, alpha: 1)

  }

}
